#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdarg.h>
#include <fcntl.h>

#include "neaacdec.h"
#include "plan9.h"

int vfd = -1;
int cfd = -1;

enum {
	Bufsiz = 4096,
};


void
outaes(ushort n)
{
	putchar(n & 0xff);
	putchar((n >> 8) & 0xff);
}

int
play(NeAACDecHandle dec, int fd, char *file)
{
 	ulong rate;
	ushort *out;
	char tmp[64];
	int i, got, len, off;
	uchar *buf, chans;
    NeAACDecFrameInfo inf;
    NeAACDecConfigurationPtr cfg;

	buf = calloc(Bufsiz, 1);
	if(buf == NULL)
		sysfatal("No memory - %r\n");

    cfg = NeAACDecGetCurrentConfiguration(dec);
    cfg->defSampleRate = 44100;
    cfg->defObjectType = LC;
    cfg->outputFormat = FAAD_FMT_16BIT;
    cfg->downMatrix = 0;
    cfg->useOldADTSFormat = 0;
	cfg->dontUpSampleImplicitSBR = 0;
    NeAACDecSetConfiguration(dec, cfg);

	if((got = read(fd, buf, Bufsiz)) <= 0)
		sysfatal("read - unexpected EOF - %r\n");

	if(NeAACDecInit(dec, buf, got, &rate, &chans) < 0)
		sysfatal("NAACDecInit() failed\n");

	if(vfd < 0)
		fprintf(stderr, "warning: audio sample rate is %d Hz\n", rate);
	else{
		snprintf(tmp, sizeof(tmp), "speed %d\n", rate);
		write(vfd, tmp, strlen(tmp));
		snprintf(tmp, sizeof(tmp), "rate %d\n", rate);
		write(cfd, tmp, strlen(tmp));
	}

	len = got;
	while(len > 0){
		out = NeAACDecDecode(dec, &inf, buf, len);
		if(inf.error != 0){
			sysfatal("%s NAACDecInit() failed, %s\n", file, NeAACDecGetErrorMessage(inf.error));
			break;
		}

		for(i = 0; i < inf.samples; i++){
			outaes(out[i]);
			switch(inf.channels){
				case 1:	break;			/* mono */
				case 2: i++; break;		/* stereo */
				default: sysfatal("%s, %d bad channel count\n", file, inf.channels);
			}
			outaes(out[i]);
		}
		

		memmove(buf, buf+inf.bytesconsumed, len-inf.bytesconsumed);
		len -= inf.bytesconsumed;

		do{
			if((got = read(fd, buf+len, Bufsiz-len)) <= 0)
				goto eof;
			len += got;
		}while(Bufsiz - len > (Bufsiz / 8));
	}
eof:
	return 0;
}


void
usage(void)
{
	fprintf(stderr, "%s [-o outfile] [file...]\n", argv0);
	exit(1);
}

int
main(int argc, char *argv[])
{
	char *outfile;
	int o, i, fd;
    NeAACDecHandle dec;
	
	argv0 = argv[0];
	outfile = NULL;
	while((o = getopt(argc, argv, "o:")) != -1)
		switch(o){
		case 'o':
			outfile = optarg;
			break;
		default:
			usage();
		}

	if(outfile){
		if((fd = creat(outfile, 0644)) < 0)
			sysfatal("%s cannot create - %r\n", outfile);
	}else{
		if((fd = open("/dev/audio", O_WRONLY)) < 0)
			sysfatal("/dev/audio cannot open - %r\n");
		vfd = open("/dev/volume", O_WRONLY);
		cfd = open("/dev/audioctl", O_WRONLY);
	}
	if(fd != 1){
		dup2(fd, 1);
		close(fd);
	}

    dec = NeAACDecOpen();
	if(argc - optind == 0){
		play(dec, 0, "standard input");
	}
	else{
		for(i=optind; i<argc; i++){
			if((fd = open(argv[i], O_RDONLY)) < 0)
				sysfatal("%s cannot open - %r\n", argv[i]);
			play(dec, fd, argv[i]);
			close(fd);
		}
	}
	NeAACDecClose(dec);
	return 0;
}
