#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdarg.h>
#include <fcntl.h>

#include "neaacdec.h"
#include "mp4ff.h"

#include "plan9.h"

int vfd = -1;
int cfd = -1;

enum {
	Bufsiz = 4096,
};

static void
outaes(ushort n)
{
	putchar(n & 0xff);
	putchar((n >> 8) & 0xff);
}

static ulong
doread(void *v, void *buf, ulong len)
{
	return fread(buf, 1, len, v);
}

static ulong
doseek(void *v, uvlong off)
{
    return fseek(v, off, SEEK_SET);
}


static int
aactrack(mp4ff_t *mp4ff)			/* find the first track index that contains valid AAC audio */
{
	uint len;
	uchar *buf;
	int i, n, rc;
	mp4AudioSpecificConfig cfg;

	n  = mp4ff_total_tracks(mp4ff);

	for(i = 0; i < n; i++){
		len = 1024;
		buf = malloc(1024);;
		mp4ff_get_decoder_config(mp4ff, i, &buf, &len);

		if(buf){
			rc = NeAACDecAudioSpecificConfig(buf, len, &cfg);
			free(buf);
			if(rc < 0)
				continue;
			return i;
		}
	}
	return -1;
}

static int
play(NeAACDecHandle dec, FILE *fp, char *file)
{
	uint len;
	ulong rate;
	ushort *out;
	char tmp[64];
	mp4ff_t *mp4ff;
	uchar *buf, chans;
	NeAACDecFrameInfo inf;
	mp4ff_callback_t *mp4cb;
	mp4AudioSpecificConfig asc;
	int i, rc, track, nsamp, s;
	NeAACDecConfigurationPtr cfg;

	cfg = NeAACDecGetCurrentConfiguration(dec);
    cfg->defSampleRate = 44100;
    cfg->defObjectType = LC;
    cfg->outputFormat = FAAD_FMT_16BIT;
    cfg->downMatrix = 0;
    cfg->useOldADTSFormat = 0;
	cfg->dontUpSampleImplicitSBR = 0;
	NeAACDecSetConfiguration(dec, cfg);

	mp4cb = malloc(sizeof(mp4ff_callback_t));
	if(mp4cb == nil)
		sysfatal("No memory %r\n");
	mp4cb->read = doread;
	mp4cb->seek = doseek;
	mp4cb->user_data = fp;

	mp4ff = mp4ff_open_read(mp4cb);
	if(mp4ff == nil)
		sysfatal("%s cannot open/read %r\n", file);

	if((track = aactrack(mp4ff)) < 0)
		sysfatal("%s no AAC sound track found in file\n", file);

	len = 0;
	buf = NULL;
	mp4ff_get_decoder_config(mp4ff, track, &buf, &len);

	if(NeAACDecInit2(dec, buf, len, &rate, &chans) < 0)
		sysfatal("cannot initialise AAC decoder\n");

	if(vfd < 0)
		fprintf(stderr, "warning: audio sample rate is %d Hz\n", rate);
	else{
		snprintf(tmp, sizeof(tmp), "speed %d\n", rate);
		write(vfd, tmp, strlen(tmp));
		snprintf(tmp, sizeof(tmp), "rate %d\n", rate);
		write(cfd, tmp, strlen(tmp));
	}

	nsamp = mp4ff_num_samples(mp4ff, track);
	for(s = 0; s < nsamp; s++){

		len = 0;
		buf = NULL;
		rc = mp4ff_read_sample(mp4ff, track, s, &buf, &len);
		if(rc == 0)
			sysfatal("%s read failed\n", file);

		out = NeAACDecDecode(dec, &inf, buf, len);
		if(inf.error != 0){
			sysfatal("%s NAACDecInit() failed, %s\n", file, NeAACDecGetErrorMessage(inf.error));
			break;
		}

		for(i = 0; i < inf.samples; i++){
			outaes(out[i]);
			switch(inf.channels){
				case 1:	break;			/* mono */
				case 2: i++; break;		/* stereo */
				default: sysfatal("%s, %d bad channel count\n", file, inf.channels);
			}
			outaes(out[i]);
		}

		free(buf);
	}
	return 0;
}


void
usage(void)
{
	fprintf(stderr, "%s [-o outfile] [file...]\n", argv0);
	exit(1);
}

int
main(int argc, char *argv[])
{
	FILE *fp;
	char *outfile;
	int o, i, fd;
    NeAACDecHandle dec;
	
	argv0 = argv[0];
	outfile = NULL;
	while((o = getopt(argc, argv, "o:")) != -1)
		switch(o){
		case 'o':
			outfile = optarg;
			break;
		default:
			usage();
		}

	if(outfile){
		if((fd = creat(outfile, 0644)) < 0)
			sysfatal("%s cannot create - %r\n", outfile);
	}else{
		if((fd = open("/dev/audio", O_WRONLY)) < 0)
			sysfatal("/dev/audio cannot open - %r\n");
		vfd = open("/dev/volume", O_WRONLY);
		cfd = open("/dev/audioctl", O_WRONLY);
	}
	if(fd != 1){
		dup2(fd, 1);
		close(fd);
	}

    dec = NeAACDecOpen();
	if(argc - optind == 0){
		play(dec, stdin, "standard input");
	}
	else{
		for(i=optind; i<argc; i++){
			if((fp = fopen(argv[i], "r")) == nil)
				sysfatal("%s cannot open - %r\n", argv[i]);
			play(dec, fp, argv[i]);
			fclose(fp);
		}
	}
	NeAACDecClose(dec);
	return 0;
}
