#include <u.h>
#include <libc.h>
#include "flac/stream_decoder.h"

static int inpfd;
static int cfd;
static int vfd;

static FLAC__StreamDecoderReadStatus
inp(FLAC__StreamDecoder *dec, FLAC__byte buf[], unsigned *bytes, void *client_data)
{
	int n = *bytes;

	n = read(inpfd, *bytes, buf);
	if(n <= 0){
		*bytes = 0;
		return FLAC__STREAM_DECODER_READ_STATUS_END_OF_STREAM;
	}

	*bytes = n;
	return FLAC__STREAM_DECODER_READ_STATUS_CONTINUE;
}

static FLAC__StreamDecoderWriteStatus
out(FLAC__StreamDecoder *dec, FLAC__Frame *frame, FLAC__int32 *buffer[], void *client_data)
{
        static int rate, chans, bits;
        static unsigned char *buf;
        static int nbuf, ifd = -1;
        FLAC__int32 *s, v;
        unsigned char *p;
        int i, j, n, b, len;

        len = frame->header.blocksize;
        b = (bits+7)/8;
        n = b * chans * len;
        if(n > nbuf){
                nbuf = n;
                buf = realloc(buf, nbuf);
                if(buf == NULL)
                    sysfatal("No memory %r\n");
        }
        p = buf;
        for(j=0; j < chans; j++){
                s = buffer[j];
                p = buf + j*b;
                for(i=0; i < len; i++){
                        n = 0;
                        v = *s++;
                        switch(b){
                        case 4:
                                p[n++] = v, v>>=8;
                        case 3:
                                p[n++] = v, v>>=8;
                        case 2:
                                p[n++] = v, v>>=8;
                        case 1:
                                p[n] = v;
                        }
                        p += chans*b;
                }
        }
        n = b * chans * len;
        if(n > 0){
                if(write(1, buf, n) != n)
					sysfatal("write failed %r\n");
		}
        return FLAC__STREAM_DECODER_WRITE_STATUS_CONTINUE;
}

static void
err(FLAC__StreamDecoder *dec, FLAC__StreamDecoderErrorStatus status, void *client_data)
{
	sysfatal("flac decoder error %s\n", FLAC__StreamDecoderErrorStatusString[status]);
}

static void
meta(FLAC__StreamDecoder *dec, FLAC__StreamMetadata *metadata, void *client_data)
{

	if(metadata->type != FLAC__METADATA_TYPE_STREAMINFO)
		return;
			
	if(metadata->data.stream_info.channels != 2)
		sysfatal("not 2 channel audio\n");

	if(metadata->data.stream_info.bits_per_sample != 16)
		sysfatal("not 16bit audio\n");

	if(metadata->data.stream_info.sample_rate != rate)
		if(vfd < 0)
			fprint(2, "warning: audio sample rate is %d Hz\n", rate);
		else{
			fprint(vfd, "speed %d\n", rate);
			fprint(cfd, "rate %d\n", rate);		/* for annoying ac97 driver, till its fixed */
		}
	}
}


int
play(int fd)
{
	FLAC__bool ok = true;
	FLAC__StreamDecoder *dec = 0;

	inpfd = fd;
	dec = FLAC__stream_decoder_new();
	FLAC__stream_decoder_set_read_callback(dec, inp);
	FLAC__stream_decoder_set_write_callback(dec, iout);
	FLAC__stream_decoder_set_error_callback(dec, err);
	FLAC__stream_decoder_set_metadata_callback(dec, meta);
	FLAC__stream_decoder_init(dec);
	FLAC__stream_decoder_process_until_end_of_stream(dec);
	FLAC__stream_decoder_finish(dec);
	return 0;
}

void
main(int argc, char **argv)
{
	int i, fd;
	char *p;

	ARGBEGIN{
	case 'o':
		outfile = EARGF(usage());
		break;
	default:
		usage();
	}ARGEND
	
	if(outfile){
		if((fd = create(outfile, OWRITE, 0666)) < 0)
			sysfatal("create %s: %r", outfile);
	}else{
		if((fd = open("/dev/audio", OWRITE)) < 0)
			sysfatal("open /dev/audio: %r");
		vfd = open("/dev/volume", OWRITE);
		cfd = open("/dev/audioctl", OWRITE);
	}
	if(fd != 1){
		dup(fd, 1);
		close(fd);
	}

	if(argc == 0){
		play(0);
	}else{
		for(i=0; i<argc; i++){
			if((fd = open(argv[i], OREAD)) < 0)
				sysfatal("open %s: %r", argv[i]);
			play(fd);
			close(fd);
		}
	}
	exits(0);
}
