#include <u.h>
#include <libc.h>
#include <bio.h>
#include <ctype.h>
#include "dat.h"
#include "fns.h"

static Item *
alloc(Item *i)
{
	if(i)
		return i;
	i = mallocz(sizeof(Item), 1);
	if(i == nil){
		werrstr("m3u: No memory for another item - %r");
		return nil;
	}
	return i;
}

Item *
m3u(int fd)
{
	Biobuf bin;
	char *line, buf[1024];
	Item *root, *i, **prev;

	i = nil;
	root = nil;
	prev = &root;
	Binit(&bin, fd, OREAD);
	while(Bgetline(&bin, buf, sizeof(buf)) != -1){
		line = trim(buf);

		for(; isspace(*line); line++)				/* strip leading whitespace */
			continue;	
		if(*line == 0)								/* blank lines */
			continue;

		if(cistrncmp(line, "#EXTINF:", 8) == 0){	/* structured comment, extended info */
			line += 8;
			i = alloc(i);
			if(i == nil)
				break;
			i->secs = strtol(line, &line, 10);
			if(*line == ',')
				line++;
			while(isspace(*line))
				line++;
			i->title = strdup(line);
			if(i->title == nil){
				werrstr("m3u: No memory for path - %r");
				free(i->path);
				free(i);
				break;
			}
		}

		if(*line == '#')							/* normal comment */
			continue;

		if(i == nil){
			i = alloc(i);
			if(i == nil)
				break;
		}
		i->path = strdup(line);
		if(i->path == nil){
			werrstr("m3u: No memory for path - %r");
			free(i->title);
			free(i);
			break;
		}

		i->next = *prev;
		*prev = i;
		prev = &i->next;
		i = nil;
	}
	Bterm(&bin);
	return root;
}
