/*
 * fetch and parse the metadata meant for the Palm7 device.
 * somtimes this is the only way to get the artist/song info
 */
#include <u.h>
#include <libc.h>
#include <bio.h>
#include <xml.h>
#include "dat.h"
#include "fns.h"

typedef struct Live365 Live365;
struct Live365 {
	int pid;
	QLock;
	int updated;
	char title[1024];
	char artist[1024];
	char album[1024];
	int length;
};

static void
poll(Sess *s, char *url, Live365 *l36)
{
	Xml *xp;
	Elem *ep;
	int fd, len, refresh;

	refresh = 60;
	for(; 1; sleep(refresh*1000)){
		if((fd = httpredial(s, 1, "9radio", url, nil)) < 0){
			continue;
		}

		if((xp = Xparse(fd, 1024*10, Fcrushwhite|Fstripnamespace)) == nil){
			fprint(2, "parse of lastfm RSS feed failed\n");
			close(fd);
			continue;
		}
		close(fd);

		if((ep = Xfind(xp, xp->root, "Playlist/Refresh")) != nil){
			if(ep->pcdata){
				refresh = atoi(ep->pcdata);
				if(refresh < 10 || refresh > 120)
					refresh = 60;
			}
		}

		qlock(l36);
		if((ep = Xfind(xp, xp->root, "Playlist/PlaylistEntry/Title")) != nil){
			if(strncmp(l36->title, ep->pcdata, sizeof(l36->title)) != 0){
				snprint(l36->title, sizeof(l36->title), "%s", ep->pcdata);
				l36->updated = 1;
			}
		}

		if((ep = Xfind(xp, xp->root, "Playlist/PlaylistEntry/Artist")) != nil){
			if(ep->pcdata && strncmp(l36->artist, ep->pcdata, sizeof(l36->artist)) != 0){
				snprint(l36->artist, sizeof(l36->artist), "%s", ep->pcdata);
				l36->updated = 1;
			}
		}

		if((ep = Xfind(xp, xp->root, "Playlist/PlaylistEntry/Album")) != nil){
			if(ep->pcdata && strncmp(l36->album, ep->pcdata, sizeof(l36->album)) != 0){
				snprint(l36->album, sizeof(l36->album), "%s", ep->pcdata);
				l36->updated = 1;
			}
		}

		if((ep = Xfind(xp, xp->root, "Playlist/PlaylistEntry/Seconds")) != nil){
			if(ep->pcdata){
				len = atoi(ep->pcdata);
				if(len != l36->length){
					l36->length = len;
					l36->updated = 1;
				}
			}
		}
		qunlock(l36);
	
		Xfree(xp);
	}
}

void *
x_open(Sess *s, char *url)
{
	Live365 *l36;

	if(url == nil || *url == 0)
		return nil;

	if((l36 = mallocz(sizeof(Live365), 1)) == nil){
		fprint(2, "No memory %r\n");
		return nil;
	}

	l36->pid = rfork(RFPROC|RFFDG|RFMEM);
	switch(l36->pid){
	case -1:			/* fail */
		free(l36);
		return nil;
	case 0:				/* child */
		poll(s, url, l36);
		exits(0);
	default:			/* parent */
		break;
	}
	return l36;
}

static void
x_sync(void *v)
{
	Live365 *l36 = v;
	if(l36 == nil)
		return;

	qlock(l36);
	if(l36->updated){
		Bprint(Meta, "title %q\n", l36->title);
		Bprint(Meta, "artist %q\n", l36->artist);
		Bprint(Meta, "album %q\n", l36->album);
		Bprint(Meta, "length %d:%02d\n", l36->length/60, l36->length%60);
		l36->updated = 0;
	}
	qunlock(l36);
}

static void
x_close(void *v)
{
	Waitmsg *w;
	Live365 *l36 = v;
	if(l36 == nil)
		return;

	postnote(PNPROC, l36->pid, "die");
	while((w = wait()) != nil)
		if(w->pid == l36->pid)
			break;
	free(w);
	free(l36);
	return;
};

Metastyle meta_live365[] = {
	.name = "live365",
	.open = x_open,
	.sync = x_sync,
	.close = x_close,
};
