#include <u.h>
#include <libc.h>
#include <bio.h>
#include <ctype.h>
#include "dat.h"
#include "fns.h"

static char *
getval(char *s)
{
	char *p, *e;

	for(p = s; *p && *p != '='; p++)
		continue;
	if(*p != '='){
		werrstr("%s - no '=' found", s);
		return nil;
	}
	p++;

	while(isspace(*p))
		p++;

	if(*p == '"' || *p == '\''){
		for(e = p; *e && *e != *p; e++)
			continue;
		if(*e != 0){
			werrstr("%s - unmatched quote", s);
			return nil;
		}
		*e = 0;
		p++;
	}
	return p;
}

static Item *
grow(Item **root, char *s)
{
	int n;
	Item *i, **prev;

	n = atoi(s);
	for(i = *root; i; i = i->next)
		if(i->slot == n)
			break;

	if(i == nil){
		/* NB: insert in correct place in sorted list */
		prev = root;
		for(i = *root; i; i = i->next){
			if(i->slot > n)
				break;
			prev = &i->next;
		}
		if((i = mallocz(sizeof(Item), 1)) == nil){
			werrstr("No more memory for a bigger index - %r");
			return nil;
		}
		i->slot = n;

		i->next = *prev;
		*prev = i;
	}

	return i;	
}


Item *
pls(int fd)
{
	Biobuf bin;
	Item *root, *i;
	char *p, *line, buf[1024];

	root = nil;
	Binit(&bin, fd, OREAD);
	while(Bgetline(&bin, buf, sizeof(buf)) != -1){

		line = trim(buf);
		if(cistrncmp(line, "Title", 5) == 0){
			i = grow(&root, line+5);
			if(i == nil)
				break;
			p = getval(line+5);
			if(p == nil)
				break;
			i->title = strdup(p);
			if(i->title == nil)
				break;
		}
		if(cistrncmp(line, "File", 4) == 0){
			i = grow(&root, line+4);
			if(i == nil)
				break;
			p = getval(line+4);
			if(p == nil)
				break;
			i->path = strdup(p);
			if(i->path == nil)
				break;
		}
		if(cistrncmp(line, "Length", 6) == 0){
			i = grow(&root, line+6);
			if(i == nil)
				break;
			p = getval(line+6);
			if(p == nil)
				break;
			i->secs = strtol(p, nil, 10);
		}
	}
	Bterm(&bin);
	return root;
}
