#include <u.h>
#include <libc.h>
#include <bio.h>
#include <ctype.h>
#include "dat.h"
#include "fns.h"

/*
 * Here we take care to read streams which have valid data on
 * their last line but are missing the last newline character.
 */
int
Bgetline(Biobuf *b, char *buf, int nbuf)
{
	int i, rv;
	Rune r;

	for(i = 0; i < nbuf - UTFmax - 1; ){
		rv = Bgetrune(b);
		if(rv == -1){
			if(i == 0)
				return -1;
			break;
		}
		if(rv == '\n')
			break;
		r = rv;
		i += runetochar(buf+i, &r);
	}
	buf[i] = 0;
	return i;
}

int
setprio(int pid, int prio)
{
	int fd;
	char buf[128];

	snprint(buf, sizeof(buf), "/proc/%d/ctl", pid);
	if((fd = open(buf, OWRITE)) == -1)
		return -1;
	fprint(fd, "pri %d\n", prio);
	close(fd);
	return 0;
}

char *
trim(char *s)
{
	char *p;

	for(; isspace(*s); s++)
		continue;	
	for(p = strchr(s, 0) -1; p >= s && isspace(*p); p--)
		*p = 0;
	return s;
}

int
httpredial(Sess *s, int conn, char *agent, char *url, char *etag)
{
	int e, n, fd, tries;
	char buf[128], *a[8], err[ERRMAX];

	tries = 0;
	if(Debug)
		fprint(2, "dialing %q as %q\n", url, agent);
Retry:
	snprint(buf, sizeof buf, "%s/%d/ctl", s->webfs, s->conid[conn]);
	if((fd = open(buf, OWRITE)) < 0){
		werrstr("cannot open another HTTP session - %r");
		return -1;
	}
	fprint(fd, "url %s", url);
	fprint(fd, "headers icy-metadata: 1");
	fprint(fd, "headers User-agent: %s", agent);
	if(etag && *etag)
		fprint(fd, "headers If-None-Match: %s", etag);
	close(fd);

	snprint(buf, sizeof buf, "%s/%d/body", s->webfs, s->conid[conn]);
	fd = open(buf, OREAD);

	if(fd < 0){
		rerrstr(err, sizeof(err));
		if(Debug)
			fprint(2, "connect failed err=%s\n", err);

		/*
		 * Wait whilst the socket closes, usually less than a second.
		 * We could use 2 sockets and ping pong between them but this
		 * is good enough for now.
		 */
		if(strstr(err, "client in use") != nil && tries++ < 10){
			sleep(300);
			goto Retry;
		}

		if((n = tokenize(err, a, nelem(a))) >= 3){
			e = atoi(a[1]);
			if(e == 0 && strcmp(a[n-1], "interrupted") == 0)
				werrstr("timeout on connect");
			else
				httperrstr(e);
		}
		return -1;
	}
	return fd;
}

int
httpdial(Sess *s, int conn, char *agent, char *url)
{
	return httpredial(s, conn, agent, url, nil);
}


int
httphdr(Sess *s, int conn, char *field, char *val, int len)
{
	int n, fd;
	char buf[128];

	if(val && len > 0)
		*val = 0;

	snprint(buf, sizeof(buf), "%s/%d/%s", s->webfs, s->conid[conn], field);
	if((fd = open(buf, OREAD)) < 0)
		return -1;
	n = read(fd, val, len-1);
	close(fd);

	if(n < 0)
		return -1;
	val[n] = 0;
	return 0;
}

