#include <u.h>
#include <libc.h>
#include <bio.h>

/* http://www.sonicspot.com/guide/wavefiles.html */

enum {
	FmtPCM = 1,
	FmtFloat = 3,
	FmtAlaw = 6,
	FmtMulaw = 7,
	FmtExt = 0xfffe,

	MaxFileSz = 4294967040,
};

char *
fmtname(int n)
{
	switch(n){
	case 1:		return "integer PCM";
	case 3:		return "float PCM";
	case 6:		return "A-law PCM";
	case 7:		return "µ-law PCM";
	default:	break;
	}
	return "unknown";
}

int
rd(Biobuf *bi, int wid)
{
	int i, n;
	uint val;

	val = 0;
	for(i = 0; i < wid; i++){
		if((n = Bgetc(bi)) < 0)
			sysfatal("unexpected EOF on input\n");
		val |= n << i*8;
	}

	return val;
}

void
wavinfo(Biobuf *bo, Biobuf *bi)
{
 	int rate, fmt, depth, freq, chans;

	if(rd(bi, 4) != 0x46464952)		/* RIFF */
		sysfatal("missing 'RIFF' chunk\n");

	rd(bi, 4);						/* chunk size, total file size - 8 */
	if(rd(bi, 4) != 0x45564157)		/* format ID, "WAVE" */
		sysfatal("missing 'WAVE' chunk\n");

	if(rd(bi, 4) != 0x20746d66)		/* subchunk ID, "fmt " */
		sysfatal("missing 'fmt ' chunk\n");

	rd(bi, 4);						/* subchunk size */
	fmt = rd(bi, 2);				/* audio format */ 
	chans = rd(bi, 2);				/* number of channels */
	freq = rd(bi, 4);				/* sample rate */
	rate = rd(bi, 4)*8;			/* byte rate */
	rd(bi, 2);						/* block align */
	depth = rd(bi, 2);				/* bits per sample */

	Bprint(bo, "wav.fmt %q\n", fmtname(fmt));
	Bprint(bo, "wav.chans %d\n", chans);
	Bprint(bo, "wav.depth %dbps\n", depth);
	Bprint(bo, "wav.freq %gKHz\n", freq/1000.0);
	Bprint(bo, "wav.rate %dKbps\n", rate/1024);

}

static void
usage(void)
{
	fprint(2, "%s [file]\n", argv0);
	exits("usage");
}

void
main(int argc, char *argv[])
{
	Biobuf *bi, *bo, out, inp;

	ARGBEGIN{
	default:
		usage();
		break;
	}ARGEND;

	if(argc > 1)
		usage();

	quotefmtinstall();
	Binit(&out, 1, OWRITE);
	bo = &out;

	if(argc == 0){
		Binit(&inp, 0, OREAD);
		bi = &inp;
	}
	else{
		if((bi = Bopen(argv[0], OREAD)) == nil)
			sysfatal("%s cannot open %r\n", argv[0]);
	}

	wavinfo(bo, bi);
	exits(nil);
}
